function changeName(type, char_obj, str)
    if(str ~= nil) then
        local name_id = char_obj:getName() -- get the identifier name of the object

--[[
    All the new names (and the default ones of those objects that got a new name)
    have to be stored in global variables, because otherwise they won't be saved
    in savegames.
]]

        -- 1. Renaming object is character
        if type == 'char' then
            if char_names == nil then char_names = {} end
            if char_orig_names == nil then char_orig_names = {} end

            -- Save the default names in a global table, using the identifier name as identifier
            if char_orig_names[name_id] == nil then
                char_orig_names[name_id] = char_obj.Name.TextLanguages
            end

            -- Save new names in global table
            char_names[name_id] = str.TextLanguages
        else
            -- 2. Renaming object is item
            if char_obj.IsItem then
                if item_names == nil then item_names = {} end
                if item_orig_names == nil then item_orig_names = {} end

                -- Save the default names in a global table, using the identifier name as identifier
                if item_orig_names[name_id] == nil then
                    item_orig_names[name_id] = char_obj.Name.TextLanguages
                end

                -- Save new names in global table
                item_names[name_id] = str.TextLanguages
            else
                -- 3. Renaming object is scene object
                local scene_found = false
                local scene_name = ''

                -- Find the name of the scene the object belongs to
                for k, scene in ipairs(Scenes) do
                    for i, scene_obj in ipairs(scene.Objects) do
                        if scene_obj:getId().id == char_obj:getId().id then
                            scene_name = scene:getName()

                            scene_found = true
                            break
                        end
                    end

                    if scene_found then break end
                end

                if scene_name ~= '' then
                    if obj_names == nil then obj_names = {} end
                    if obj_names[scene_name] == nil then obj_names[scene_name] = {} end
                    if obj_orig_names == nil then obj_orig_names = {} end
                    if obj_orig_names[scene_name] == nil then obj_orig_names[scene_name] = {} end

                    -- Save the default names in a global table, using the scene name and the identifier name as identifiers
                    if obj_orig_names[scene_name][name_id] == nil then
                        obj_orig_names[scene_name][name_id] = char_obj.Name.TextLanguages
                    end

                    -- Save new names in global table
                    obj_names[scene_name][name_id] = str.TextLanguages
                end
            end
        end

        -- Set the new names of the object
        char_obj.Name.TextLanguages = str.TextLanguages
    end
end


--[[
    The reload function gets all entries from the global variables and assigns them
    back to the objects. It has to be executed after loading a savegame.
]]

function reloadResetNames(switch)
    if switch == 1 then
        -- If "reset" option selected, loop through the three global tables with default names
        -- and assign them to the objects; delete the global table of new names
        if char_orig_names ~= nil then
            for k, v in pairs(char_orig_names) do
                Characters[k].Name.TextLanguages = char_orig_names[k]
                char_names[k] = nil
            end
        end
        if item_orig_names ~= nil then
            for k, v in pairs(item_orig_names) do
                game.Items[k].Name.TextLanguages = item_orig_names[k]
                item_names[k] = nil
            end
        end
        if obj_orig_names ~= nil then
            for k, v in pairs(obj_orig_names) do
                for n, w in pairs(v) do
                    Scenes[k].Objects[n].Name.TextLanguages = v[n]
                    obj_names[n] = nil
                end
                
                obj_names[k] = nil
            end
        end
    else
        -- If "reload" option selected, loop through the three global tables of new names
        -- and assign them to the objects
        if char_names ~= nil then
            for k, v in pairs(char_names) do
                Characters[k].Name.TextLanguages = char_names[k]
            end
        end
        if item_names ~= nil then
            for k, v in pairs(item_names) do
                game.Items[k].Name.TextLanguages = item_names[k]
            end
        end
        if obj_names ~= nil then
            for k, v in pairs(obj_names) do
                for n, w in pairs(v) do
                    Scenes[k].Objects[n].Name.TextLanguages = v[n]
                end
            end
        end
    end
end




-- Register action parts

registerActionPart('CHANGE_CHARACTER_NAME', function(activeAction, actionPart)
    changeName('char', actionPart.AltLink, actionPart.Link)
    return false
end)

registerActionPart('CHANGE_OBJECT_NAME', function(activeAction, actionPart)
    changeName('obj', actionPart.AltLink, actionPart.Link)
    return false
end)

registerActionPart('RELOAD_RESET_NAMES', function(activeAction, actionPart)
    reloadResetNames(actionPart.Int)
    return false
end)